<?php
/**
 * Created by JetBrains PhpStorm.
 * User: leonardooliveira
 * Date: 10/07/14
 * Time: 11:43
 * To change this template use File | Settings | File Templates.
 */

namespace Financeiro\Logic\ContasReceber\Cnab\Retorno;

use DateTime;
use Financeiro\Entities\Boletreceber;
use Geral\Util\Ambiente;

class LeitorRetornoCaixa extends LeitorRetorno{

    const ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    const ID_TP_ARQUIVO_REGISTRO_HEADER = '1';
    const ID_TP_ARQUIVO_DETALHE = '3';
    const CODIGO_COMPENSACAO_CAIXA = '104';

    private $arquivo;
    /** @var $retorno Retorno */
    private $retorno;

    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco){
        try {
            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function leArquivo(){
        try {
            foreach ($this->arquivo as $linha) {
                $cont = $this->contlinha++;

                $idsegmento = substr($linha, 13, 1);
                if($idsegmento == 'U')
                    $cont = $cont - 1;
                else
                    $this->retorno = new Retorno();


                // Busca o Tipo de Registro do Arquivo
                $idRegistro = $linha[0];

                switch ($idRegistro) {

                    case(self::ID_TP_ARQUIVO_HEADER_ARQUIVO):
                        self::leHeaderArquivo($linha);
                        break;

                    case(self::ID_TP_ARQUIVO_REGISTRO_HEADER):
                        self::leHeaderLote($linha);
                        break;
                    case(self::ID_TP_ARQUIVO_DETALHE):
                        self::leDetalhe($linha);
                        break;
                }
                $this->retornos[$cont] = $this->retorno;
            }
            self::preparaOcorrencias($this->cdbanco);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    //########################################################################
    //############# Identificação do Registro Header #########################
    //########################################################################
    private function leHeaderArquivo($header){

        // CAMPO: Código do BCO na compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if(substr($header, 0, 3) != self::CODIGO_COMPENSACAO_CAIXA)
            self::gravaErros($this->contlinha, 'Banco Inválido.', true);

        // CAMPO: Identificação do Arquivo de Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        if (substr($header, 142, 1) == '2'){
            self::gravaErros($this->contlinha, 'Arquivo inválido.', true);
        }
    }

    //########################################################################
    //############# Identificação do Registro Header de Lote #################
    //########################################################################
    private function leHeaderLote($header){

        // CAMPO: Tipo de Inscrição da empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $tpinscricao = trim(substr($header, 17, 1));
        if (empty($tpinscricao)){
            self::gravaErros($this->contlinha, 'Tipo de inscrição da empresa inválido.', true);
        }


        // CAMPO: Inscrição da empresa
        // POSIÇÃO: 019 a 033
        // PICTURE: 9(15)
        $inscricao = trim(substr($header, 18, 15));
        if (empty($inscricao)){
            self::gravaErros($this->contlinha, 'Inscrição da empresa inválida.', true);
        }
    }

    //########################################################################
    //############# Identificação do Registro Detalhe ########################
    //########################################################################
    private function leDetalhe($detalhe){
        $validaOcorre = substr($detalhe, 15, 02);
        if($validaOcorre != '28'){
            //############# Identificação do Registro de Segmento T
            if(substr($detalhe, 13, 1) == 'T'){
                $cdfilial = substr($detalhe, 105, 4);
                $nrlanbolrec = substr($detalhe, 109, 10);
                $idtitulobanco = trim(substr($detalhe, 46, 11));
                $idcartbol = substr($detalhe, 57, 1);
                $cdempresa = tk_get_context_vars('CDEMPRESA', 'win_pbi00110_000');
                $nossonumero = Boletreceber::getRepository();
                /** @var $nossonumero \Financeiro\Repositories\Boletreceber */
                $cdnosnumrec = $nossonumero->getNossoNumero($idtitulobanco, $cdempresa);
                $boleto = false;
                if(!empty($cdfilial) && !empty($nrlanbolrec))
                    /** @var $boleto Boletreceber */
                    $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, Ambiente::getNrorg());
                elseif($cdnosnumrec)
                    /** @var $boleto Boletreceber */
                    $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                        $cdnosnumrec[0]['NRLANBOLREC'], $cdnosnumrec[0]['CDFILIAL'], Ambiente::getNrorg()
                    );
                $this->retorno->setBoleto('BOLETO');
                $this->retorno->setCdlinha(str_pad($this->contlinha, 4, 0, STR_PAD_LEFT));
                $this->retorno->setCdfilial($cdfilial);
                $this->retorno->setNrlanbolrec($nrlanbolrec);
                $this->retorno->setNossonumero($idtitulobanco);
                $this->retorno->setCarteira($idcartbol);
                $this->retorno->setCdagencia(trim(substr($detalhe, 17, 6)));
                $this->retorno->setCdconta(trim(substr($detalhe, 23, 13)));

                if($boleto) {
                    $this->retorno->setVrfinalbrec($boleto->getVrfinalbrec());
                    $this->retorno->setVrbolrec($boleto->getVrbolrec());
                    $this->retorno->setIdstatrec($boleto->getIdstatbrec());
                    $this->retorno->setCdfilial($boleto->getCdfilial());
                    $this->retorno->setNrlanbolrec($boleto->getNrlanbolrec());
                    $this->retorno->setCdtpbaixapag($boleto->getCdtpbaixarec());
                    $this->retorno->setCdreducont($boleto->getCdreducont());
                }

                // CAMPO: Identificação do título no banco
                // POSIÇÃO: 047 a 057
                // PICTURE: 9(11)
                if (empty($nossonumero))
                    $this->gravaErros($this->contlinha, 'Nosso número inválido.');

                // CAMPO: Identificação da Ocorrência no Banco
                // POSIÇÃO: 016 a 017
                // PICTURE: 9(2)
                $idocorrencia = trim(substr($detalhe, 15, 2));
                if(empty($idocorrencia))
                    $this->gravaErros($this->contlinha, 'Identificação da Ocorrência do banco.');

                // CAMPO: Valor nominal do título
                // POSIÇÃO: 082 a 096
                // PICTURE: 9(13)V9(2)
                $this->retorno->setVrnominal(round(floatval(substr($detalhe, 81, 15)) / 100, 2));

                // CAMPO: Valor de Tarifas / Custas
                // POSIÇÃO: 199 a 213
                // PICTURE: 9(13)V9(2)
                $this->retorno->setVrdesconto(round(floatval(substr($detalhe, 198, 15)) / 100, 2));

                /*doVrDesconto := 0;
                //Não considera os valores de tarifas como ajuste do boleto
                if cdsRetornoVRDESCONTO.Value > 0 then begin
                    doVrDesconto := RoundDec(cdsRetornoVRDESCONTO.Value, 2);
                    cdsRetornoVRDESCONTO.Value := 0;
                end;*/

                // CAMPO: Erros - Registros rejeitados ou alegação do sacado
                // POSIÇÃO: 214 a 223
                // PICTURE: X(10)
                $iderros = trim(substr($detalhe, 213, 10));
            }
            else{
                //################  Registro Detalhe Segmento U ###################//
                // CAMPO: Juros/Multa/Encargos
                // POSIÇÃO: 018 a 032
                // PICTURE: 9(13)V9(2)
                // CAMPO: Valor  Líquido Creditado
                // POSIÇÃO: 093 a 107
                // PICTURE: 9(13)V9(2)
                // CAMPO: Valor de outros Creditos
                // POSIÇÃO: 123 a 137
                // PICTURE: 9(13)V9(2)
                $this->retorno->setVracrescimo(
                    round(floatval(substr($detalhe, 17, 15)) / 100, 2) +
                    round(floatval(substr($detalhe, 92, 15)) / 100, 2) +
                    round(floatval(substr($detalhe, 122, 15)) / 100, 2)
                );

                // CAMPO: Desconto concedido
                // POSIÇÃO: 033 a 047
                // PICTURE: 9(13)V9(2)
                // CAMPO: Abatimento concedito sobre o título
                // POSIÇÃO: 048 a 062
                // PICTURE: 9(13)V9(2)
                $this->retorno->setVrdesconto(
                    round(floatval(substr($detalhe, 32, 15)) / 100, 2) +
                    round(floatval(substr($detalhe, 47, 15)) / 100, 2)
                );

                // CAMPO: Valor  Líquido Creditado
                // POSIÇÃO: 093 a 107
                // PICTURE: 9(13)V9(2)
                $this->retorno->setVrpago(round(floatval(substr($detalhe, 93, 15)) / 100, 2));

                //Não considera o valor de tarifas sobre o valor total pago
                if($this->retorno->getVrdesconto() > 0)
                    $this->retorno->setVrpago($this->retorno->getVrpago() - ($this->retorno->getVrdesconto() * -1));

                // CAMPO: Valor de outras Despesas
                // POSIÇÃO: 108 a 122
                // PICTURE: 9(13)V9(2)
                if($validaOcorre != '28'){
                    $this->retorno->setVrdesconto($this->retorno->getVrdesconto() + substr($detalhe, 107, 15));
                }

                // CAMPO: Data de liquidação do boleto
                // POSIÇÃO: 138 a 145
                // PICTURE: X(8)
                /** @var $dtocorrencia DateTime */
                $dtocorrencia = substr(138, 8);
                $this->retorno->setDtocorrencia($dtocorrencia->format('d/m/Y'));

                $situacao = self::situacaoCnab($validaOcorre);
                $this->retorno->setIdsitucnabr($situacao);
                if($situacao == '90'){
                    // CAMPO: Data de efetivação do crédito
                    // POSIÇÃO: 146 a 153
                    // PICTURE: X(8)
                    /** @var $vencimento DateTime */
                    $vencimento = substr($detalhe, 145, 8);
                    $this->retorno->setDtvencimento($vencimento->format('d/m/Y'));
                }
            }
        }
    }

    /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
        if($cdocorrencia == '05' || $cdocorrencia == '09')
            return '00'; //Não Agendado
        else if($cdocorrencia == '02' or $cdocorrencia == '14' or $cdocorrencia == '19' or $cdocorrencia == '23')
            return '20'; //Retorno OK
        else if($cdocorrencia == '03' or $cdocorrencia == '26' or $cdocorrencia == '30' or $cdocorrencia == '45')
            return '01'; //Retorno com Erro
        else if($cdocorrencia == '06' or $cdocorrencia == '17')
            return '90'; //Baixado
        return '10'; //Sem retorno
    }

}