<?php
namespace Planejamento\Logic\PlanejamentoProducao;

use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;


class GeraProdAcabaLogic{
    /**
     * Função que executa a procedure de conclusão do produto acabado
     *
     * @param $nrorg          -> número da organização
     * @param $nrplanprod     -> número do planejamento
     * @param $dtretirada     -> Data da retirada
     * @param $cdoperador     -> Operador de execução
     * @param $cdfiliproc     -> Filial de processamento
     * @param $dslancamento   -> Descrição da retirada
     * @param $dslanctoestoq  -> Descrição do lançamento de estoque
     * @param $operacao       -> Operação da procedure : (G) geração (C) cancelamento das informações geradas
     * @return array|bool     -> conjunto de dados dos parâmetros de saída
     * @throws \Exception
     */
    public static function concluiProdAcabado(
        $nrorg, $nrplanprod, $dtretirada, $cdoperador,
        $cdfiliproc, $dslancamento, $dslanctoestoq, $operacao, $cdproduto
    )  {
        try {
            $procedure = new Procedure('CONCLUI_PROD_ACABADO');
            $procedure->addParam(
                new Param (
                    'P_NRPLANPROD',
                    Param::PARAM_INPUT,
                    $nrplanprod,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param (
                    'P_DTRETIRADA',
                    Param::PARAM_INPUT,
                    $dtretirada,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param (
                    'P_CDOPERADOR',
                    Param::PARAM_INPUT,
                    $cdoperador,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param (
                    'P_CDFILIPROC',
                    Param::PARAM_INPUT,
                    $cdfiliproc,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param (
                    'P_DSLANCAMENTO',
                    Param::PARAM_INPUT,
                    $dslancamento,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_DSLANCTOESTOQ',
                    Param::PARAM_INPUT,
                    $dslanctoestoq,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_OPERACAO',
                    Param::PARAM_INPUT,
                    $operacao,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_NRORG',
                    Param::PARAM_INPUT,
                    $nrorg,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_CDPRODUTO',
                    Param::PARAM_INPUT,
                    $cdproduto,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param (
                    'P_RETORNO',
                    Param::PARAM_OUTPUT,
                    NULL,
                    Param::PARAM_TYPE_STR,
                    400
                )
            );

            return $procedure->call();

        } catch (\Exception $e) {
            throw $e;
        }
    }

}